﻿//
// Copyright 2005, Captator - de danske .NET eksperter
//
// Denne demo er lavet i forbindelse med artiklen: "Generics",
// læs artiklen i klummen ".Henrik om .NET" - på www.captator.dk.
//
// Den følgende kode skal blot opfattes som eksempler, der illustrerer nogle af de muligheder, der ligger i brugen af generics.
//
// Koden er til fri afbenyttelse, finder du en fejl eller har du kommentarer til forbedringer så
// sagsøg os ikke ;->, men send i stedet en mail til info@captator.dk

namespace GenericBubbleSort
{
	partial class DemoForm : System.Windows.Forms.Form
	{
		private const int _defaultNumberOfElements = 10000;
		private int[] _testData;

		public DemoForm()
		{
			InitializeComponent();
		}

		private void DemoForm_Load(object sender, System.EventArgs e)
		{
			_testData = GenerateTestData(_defaultNumberOfElements);
			txtNumberOfElements.Text = _defaultNumberOfElements.ToString();
		}

		private void btnSortIntArray_Click(object sender, System.EventArgs e)
		{
			int[] testDataCopy = CopyTestData<int>(_testData);

			BeforeSort();

			SortUtil.SortIntArray(testDataCopy);

			AfterSort<int>(lblSortIntArray, testDataCopy);
		}

		private void btnSortIComparableArray_Click(object sender, System.EventArgs e)
		{
			System.IComparable[] testDataCopy = CopyTestData<System.IComparable>(_testData);

			BeforeSort();

			SortUtil.SortIComparableArray(testDataCopy);

			AfterSort<System.IComparable>(lblSortIComparableArray, testDataCopy);
		}

		private void btnGenericSortIComparableWithIntArray_Click(object sender, System.EventArgs e)
		{
			int[] testDataCopy = CopyTestData<int>(_testData);

			BeforeSort();

			SortUtil.GenericSortIComparable<int>(testDataCopy);

			AfterSort<int>(lblGenericSortIComparableWithIntArray, testDataCopy);
		}

		private void btnGenericSortIComparableOfT_Click(object sender, System.EventArgs e)
		{
			int[] testDataCopy = CopyTestData<int>(_testData);

			BeforeSort();

			SortUtil.GenericSortIComparableOfT<int>(testDataCopy);

			AfterSort<int>(lblGenericSortIComparableOfT, testDataCopy);
		}


		private void btnMakeTestData_Click(object sender, System.EventArgs e)
		{
			try
			{
				int numberOfElements = System.Int32.Parse(txtNumberOfElements.Text);

				if (numberOfElements <= 0)
					throw new System.Exception();

				this.Cursor = System.Windows.Forms.Cursors.WaitCursor;

				_testData = GenerateTestData(numberOfElements);

				ClearSortResults();

				this.Cursor = System.Windows.Forms.Cursors.Default;
			}
			catch
			{
				System.Windows.Forms.MessageBox.Show("Indtast venligst et heltal større end nul.", "Ugyldigt antal test elementer.", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Error);
			}
		}

		#region Måling af tidsforbrug

		private System.DateTime _startTime;

		private void BeforeSort()
		{
			this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
			_startTime = System.DateTime.Now;
		}

		private void AfterSort<T>(System.Windows.Forms.Label lbl, T[] sortedData)
		{
			System.DateTime endTime = System.DateTime.Now;

			this.Cursor = System.Windows.Forms.Cursors.Default;

			System.TimeSpan duration = endTime.Subtract(_startTime);

			lbl.Text = "Det tog " + duration.TotalMilliseconds + " millisekunder.";
		}

		# endregion

		# region TestData

		private T[] CopyTestData<T>(int[] arrToCopy)
		{
			T[] copy = new T[arrToCopy.Length];
			arrToCopy.CopyTo(copy, 0);
			return copy;
		}

		private int[] GenerateTestData(int numberOfElements)
		{
			int[] testData = new int[numberOfElements];
			System.Random ran = new System.Random();
			int testElement;

			for (int i = 0; i < numberOfElements; i++)
			{
				testElement = (int)System.Math.Floor(ran.NextDouble() * 17 * 42);
				testData[i] = testElement;
			}

			return testData;
		}

		# endregion

		private void ClearSortResults()
		{
			lblSortIntArray.Text = "";
			lblSortIComparableArray.Text = "";
			lblGenericSortIComparableOfT.Text = "";
			lblGenericSortIComparableWithIntArray.Text = "";
		}

		# region Visning af kode

		private void lnkSortIntArray_LinkClicked(object sender, System.Windows.Forms.LinkLabelLinkClickedEventArgs e)
		{
			txtImplementation.Text = "public static void SortIntArray(int[ ] numbers)\r\n{\r\n  for (int i = 0; i < numbers.Length - 1; i++)\r\n  {\r\n    for (int j = 0; j < numbers.Length - 1 - i; j++)\r\n    {\r\n      if (numbers[j + 1] < numbers[j])\r\n      {\r\n        Swap(ref numbers[j], ref numbers[j + 1]);\r\n      }\r\n    }\r\n  }\r\n}\r\n\r\npublic static void Swap(ref int obj1, ref int obj2)\r\n{\r\n  int tmp = obj2;\r\n  obj2 = obj1;\r\n  obj1 = tmp\r\n}";
			txtArguments.Text = "System.Int32[ ]";
		}

		private void lnkSortIComparableArray_LinkClicked(object sender, System.Windows.Forms.LinkLabelLinkClickedEventArgs e)
		{
			txtImplementation.Text = "public static void SortIComparableArray(System.IComparable[ ] numbers)\r\n{\r\n  for (int i = 0; i < numbers.Length - 1; i++)\r\n  {\r\n    for (int j = 0; j < numbers.Length - 1 - i; j++)\r\n    {\r\n      if (numbers[j + 1].CompareTo(numbers[j]) < 0)  //Kald af System.Int32.CompareTo(object)\r\n      {\r\n        Swap(ref numbers[j], ref numbers[j + 1]);\r\n      }\r\n    }\r\n  }\r\n}\r\n\r\npublic static void Swap(ref System.IComparable obj1, ref System.IComparable obj2)\r\n{\r\n  System.IComparable tmp = obj2;\r\n  obj2 = obj1;\r\n  obj1 = tmp\r\n}";
			txtArguments.Text = "System.IComparable[ ]";
		}

		private void lnkGenericSortIComparableWithIntArray_LinkClicked(object sender, System.Windows.Forms.LinkLabelLinkClickedEventArgs e)
		{
			txtImplementation.Text = "public static void GenericSortIComparable<T>(T[ ] numbers) where T : System.IComparable\r\n{\r\n  for (int i = 0; i < numbers.Length - 1; i++)\r\n  {\r\n    for (int j = 0; j < numbers.Length - 1 - i; j++)\r\n    {\r\n      if (numbers[j + 1].CompareTo(numbers[j]) < 0)\r\n      {\r\n        Swap<T>(ref numbers[j], ref numbers[j + 1]);\r\n      }\r\n    }\r\n  }\r\n}\r\n\r\npublic static void Swap<T>(ref T obj1, ref T obj2)\r\n{\r\n  T tmp = obj2;\r\n  obj2 = obj1;\r\n  obj1 = tmp\r\n}";
			txtArguments.Text = "System.Int32[ ]";
		}

		private void lnkGenericSortIComparableOfT_LinkClicked(object sender, System.Windows.Forms.LinkLabelLinkClickedEventArgs e)
		{
			txtImplementation.Text = "public static void GenericSortIComparableOfT<T>(T[ ] numbers) where T : System.IComparable<T>\r\n{\r\n  for (int i = 0; i < numbers.Length - 1; i++)\r\n  {\r\n    for (int j = 0; j < numbers.Length - 1 - i; j++)\r\n    {\r\n      if (numbers[j + 1].CompareTo(numbers[j]) < 0)  //Kald af System.Int32.CompareTo<T>(int)\r\n      {\r\n        Swap<T>(ref numbers[j], ref numbers[j + 1]);\r\n      }\r\n    }\r\n  }\r\n}\r\n\r\npublic static void Swap<T>(ref T obj1, ref T obj2)\r\n{\r\n  T tmp = obj2;\r\n  obj2 = obj1;\r\n  obj1 = tmp\r\n}";
			txtArguments.Text = "System.Int32[ ]";
		}

		# endregion

	}
}